#!/usr/bin/python3

import argparse
import configparser
import dataclasses
import os
import pathlib

from . import validate
from .validate import LinuxBinary, PlatformBinary, WindowsBinary


@dataclasses.dataclass
class BinaryMountSpec:
    # a "mount" specification, described as a pathsep-separated pair of paths
    # (we can't use colons across all platforms because Windows)
    file_path: pathlib.Path
    visible_path: pathlib.Path

    def __init__(self, arg: str):
        (self.file_path, self.visible_path) = map(
            pathlib.Path, arg.split(os.pathsep) if os.pathsep in arg else (arg, arg)
        )


def main() -> None:
    parser = argparse.ArgumentParser()

    parser.add_argument("validation_path", metavar="config", type=pathlib.Path)
    parser.add_argument(
        "--add-binary", dest="bin_mounts", type=BinaryMountSpec, action="append"
    )
    parser.add_argument("--match", dest="matches", type=str, action="append")

    args = parser.parse_args()

    validation_files = set()
    if args.validation_path.is_dir():
        validation_files.update(args.validation_path.rglob("*.ini"))
    else:
        validation_files.add(args.validation_path)

    candidate_targets: list[PlatformBinary] = []
    for mount in args.bin_mounts:
        loaded_bin: PlatformBinary
        if mount.file_path.suffix == ".so":
            loaded_bin = LinuxBinary(mount.file_path)
        elif mount.file_path.suffix == ".dll":
            loaded_bin = WindowsBinary(mount.file_path)
        else:
            raise ValueError(f"Unknown bin suffix {mount.file_path.suffix}")
        loaded_bin.path = mount.visible_path
        candidate_targets.append(loaded_bin)

    for validation_file in sorted(validation_files):
        header = validation_file.name
        if args.validation_path.is_dir():
            header = str(
                pathlib.PurePosixPath(validation_file.relative_to(args.validation_path))
            )

        if args.matches and not any(match in header for match in args.matches):
            continue

        print(header)

        config = configparser.ConfigParser()
        config.read(validation_file)

        entries = validate.read_config(config)

        for name, entry in entries.items():
            target = entry.get_target_match(candidate_targets)
            try:
                assert target, f"No binary match for {entry.target}"
                for key, result in entry.process(target).items():
                    print("- [OK]", key.substitute(name=name), "=", result)
            except Exception as e:
                print("- [FAIL]", name, f"({type(e).__name__}):", e)
                # traceback.print_exception(e)


if __name__ == "__main__":
    main()
